#include <QApplication>
#include <QWidget>
#include <QLabel>
#include <QPushButton>
#include <QPalette>
#include <QLayout>
#include <QPropertyAnimation>

class HarnessPrivate
{
public:
    HarnessPrivate() 
	: movingAreaSize( QSize( 360, 64 ) ), horizontalOffset( 8 ) {
    }

    QSize movingAreaSize;
    int horizontalOffset;
    QPropertyAnimation* labelAnimation;
};

class Harness : public QWidget
{
    Q_OBJECT
    Q_DECLARE_PRIVATE( Harness )

public:
    Harness();
    ~Harness();

private slots:
    void startAnimation();

private:
    HarnessPrivate* d_ptr;
};

Harness::Harness()
    : QWidget( 0 ), d_ptr( new HarnessPrivate )
{
    Q_D( Harness );

    QLabel* movableLabel;
    movableLabel = new QLabel( "Moving", this );
    movableLabel->setMargin( 4 );
    QPalette movableLabelPalette = movableLabel->palette();
    movableLabelPalette.setColor( QPalette::Window, Qt::red );
    movableLabel->setPalette( movableLabelPalette );
    movableLabel->setAutoFillBackground( true );
    int movableLabelY = ( d->movingAreaSize.height() - movableLabel->sizeHint().height() ) / 2;
    movableLabel->resize( movableLabel->sizeHint() );
    movableLabel->move( d->horizontalOffset, movableLabelY );
    
    QPushButton* animateButton = new QPushButton( "Animate" );

    QHBoxLayout* buttonLayout = new QHBoxLayout;
    buttonLayout->addStretch();
    buttonLayout->addWidget( animateButton );

    QVBoxLayout* topLayout = new QVBoxLayout;
    topLayout->addSpacing( d->movingAreaSize.height() );
    topLayout->addLayout( buttonLayout );

    setFixedWidth( d->movingAreaSize.width() );
    setLayout( topLayout );

    setFixedHeight( sizeHint().height() );

    d->labelAnimation = new QPropertyAnimation( movableLabel, "pos", this );
    d->labelAnimation->setDuration( 5 * 1000 );
    int movableLabelStartX = d->horizontalOffset;
    int movableLabelEndX = width() - movableLabel->sizeHint().width() - d->horizontalOffset;
    int movableLabelInterimX = movableLabelStartX + ( movableLabelEndX - movableLabelStartX ) * 9 / 10;
    d->labelAnimation->setKeyValueAt( 0.0, QPoint( movableLabelStartX, movableLabelY ) );
    d->labelAnimation->setKeyValueAt( 0.5, QPoint( movableLabelInterimX, movableLabelY ) );
    d->labelAnimation->setKeyValueAt( 1.0, QPoint( movableLabelEndX, movableLabelY ) );

    connect( animateButton, SIGNAL( clicked() ), SLOT( startAnimation() ) );
}

Harness::~Harness()
{
    delete d_ptr;
}

void Harness::startAnimation()
{
    Q_D( Harness );
    
    d->labelAnimation->start();
}

int main( int argc, char** argv )
{
    QApplication app( argc, argv );

    Harness harness;
    harness.show();

    return app.exec();
}

#include "movablelabel.moc"

